package org.springframework.boot.loader.archive;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URI;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Deque;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.NoSuchElementException;
import org.springframework.boot.loader.archive.Archive;
import org.springframework.boot.loader.archive.ExplodedArchive;

abstract class AbstractIterator<T> implements Iterator<T> {
  private static final Comparator<File> entryComparator = Comparator.comparing(File::getAbsolutePath);
  
  private final File root;
  
  private final boolean recursive;
  
  private final Archive.EntryFilter searchFilter;
  
  private final Archive.EntryFilter includeFilter;
  
  private final Deque<Iterator<File>> stack = new LinkedList<>();
  
  private ExplodedArchive.FileEntry current;
  
  private String rootUrl;
  
  AbstractIterator(File root, boolean recursive, Archive.EntryFilter searchFilter, Archive.EntryFilter includeFilter) {
    this.root = root;
    this.rootUrl = this.root.toURI().getPath();
    this.recursive = recursive;
    this.searchFilter = searchFilter;
    this.includeFilter = includeFilter;
    this.stack.add(listFiles(root));
    this.current = poll();
  }
  
  public boolean hasNext() {
    return (this.current != null);
  }
  
  public T next() {
    ExplodedArchive.FileEntry entry = this.current;
    if (entry == null)
      throw new NoSuchElementException(); 
    this.current = poll();
    return adapt(entry);
  }
  
  private ExplodedArchive.FileEntry poll() {
    while (!this.stack.isEmpty()) {
      while (((Iterator)this.stack.peek()).hasNext()) {
        File file = ((Iterator<File>)this.stack.peek()).next();
        if (ExplodedArchive.access$000().contains(file.getName()))
          continue; 
        ExplodedArchive.FileEntry entry = getFileEntry(file);
        if (isListable(entry))
          this.stack.addFirst(listFiles(file)); 
        if (this.includeFilter == null || this.includeFilter.matches((Archive.Entry)entry))
          return entry; 
      } 
      this.stack.poll();
    } 
    return null;
  }
  
  private ExplodedArchive.FileEntry getFileEntry(File file) {
    URI uri = file.toURI();
    String name = uri.getPath().substring(this.rootUrl.length());
    try {
      return new ExplodedArchive.FileEntry(name, file, uri.toURL());
    } catch (MalformedURLException ex) {
      throw new IllegalStateException(ex);
    } 
  }
  
  private boolean isListable(ExplodedArchive.FileEntry entry) {
    return (entry.isDirectory() && (this.recursive || entry.getFile().getParentFile().equals(this.root)) && (this.searchFilter == null || this.searchFilter
      .matches((Archive.Entry)entry)) && (this.includeFilter == null || 
      !this.includeFilter.matches((Archive.Entry)entry)));
  }
  
  private Iterator<File> listFiles(File file) {
    File[] files = file.listFiles();
    if (files == null)
      return Collections.emptyIterator(); 
    Arrays.sort(files, entryComparator);
    return Arrays.<File>asList(files).iterator();
  }
  
  public void remove() {
    throw new UnsupportedOperationException("remove");
  }
  
  protected abstract T adapt(ExplodedArchive.FileEntry paramFileEntry);
}
